local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local Boss = RW.bossPrototype
local iconProt = {}
local mt = {__index = iconProt}

local Party = RW:GetPlugin("Raid")
local BITS = RW.BITS

Boss.BITS = Boss.BITS or {}
Boss.BITS.ICON = {
	STAR	= 0x00000001,
	CIRCLE 	= 0x00000002,
	DIAMOND	= 0x00000004,
	TRIANGLE = 0x00000008,
	MOON 	= 0x00000010,
	SQUARE 	= 0x00000020,
	CROSS 	= 0x00000040,
	SKULL 	= 0x00000080,
}

local IconToNum  = {
	STAR	= 1,
	CIRCLE 	= 2,
	DIAMOND	= 3,
	TRIANGLE = 4,
	MOON 	= 5,
	SQUARE 	= 6,
	CROSS 	= 7,
	SKULL 	= 8,
}

local iconCoords = {
	STAR		= {0, 		0.25,	0, 		0.25},
	CIRCLE 		= {0.25,	0.5,	0,		0.25},
	DIAMOND		= {0.5, 	0.75,	0,		0.25},
	TRIANGLE 	= {0.75,	1,		0,		0.25},
	MOON 		= {0,		0.25,	0.25,	0.5},
	SQUARE 		= {0.25,	0.5,	0.25,	0.5},
	CROSS 		= {0.5,		0.75,	0.25,	0.5},
	SKULL 		= {0.75,	1,		0.25,	0.5},
}

function Boss:_Icon(defaultIcon, optionName, timeOut, ...)	
	local icons = {}
	for i = 1, select("#", ...) do
		local ic = slect(i, ...)
		icons[ic] = true
	end
	obj = setmetatable({
		type = defaultIcon,
		icon = defaultIcon,
		icons = icons,
		usedIcons = {},
		id = optionName,
		timeOut = timeOut,
		oldIcons = {},
		scheduled = {},
		mod = self,
	}, mt)
	
	self.icons = self.icons or {}
	tinsert(self.icons, obj)
	return obj
end

function Boss:Icon(defaultIcon, optionName, default, timeOut, ...)
	if not optionName then optionName = "Default" end
	if type(optionName) == "number" then
		self:AddSpellOption(optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName)
	end
	
	-- TODO: Change the GUIWidget to show all icons in one and dont add one option per icon
	-- Rather add one option that shows all icons
	for i = 1, select("#", ...) do
		local n = select(i, ...)
		if type(n) == "string" and self.BITS.ICON[n] then
			self:AddDefaults(optionName, "ICON", n, default)
			self.usedIcons = self.usedIcons or {}
			self.usedIcons[n] = true
		end
	end
	
	return self:_Icon(defaultIcon, optionName, timeOut)
end

-- Reset all icons, both reset the cache and put old marks back on people
function Boss:ResetAllIcons()
	if self.icons then
		for i, icon in pairs(self.icons) do
			for target, oldIcon in pairs(icon.oldIcons) do
				SetRaidTarget(target, oldIcon)
			end
			wipe(icon.oldIcons)
			icon:ResetIcons()
		end
	end
end

-- Puts the old mark back on people or just hides it if they did not have one when they got marked
local function hideIcon(self, target)
	SetRaidTarget(target, self.oldIcons[target] or 0)
	self.oldIcons[target] = nil
end

-- Reset the icon cache
function iconProt:ResetIcons()
	for icon in pairs(self.usedIcons) do
		self.icons[icon] = true
	end
	wipe(self.usedIcons)
end

function iconProt:Show(target, time, icon, ...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.ICON) ~= RW.BITS.ICON then return end
	if bit.band(self.mod.db.profile[self.id].ICON, self.mod.BITS.ICON[icon or self.type]) ~= self.mod.BITS.ICON[icon or self.type] then return end
	if not target then return end	
	
	local targetID = Boss:UIDFromRandom(target)
	
	if targetID and #self.icons > 0 then
		self.oldIcons[targetID] = GetRaidTargetIndex(targetID) or 0
		
		local icn
		if icon then
			if self.icons[icon] then
				icn = icon
				self.usedIcons[icon] = true
				self.icons[icon] = false
			end
		else
			icn = next(self.icons)
			self.icons[icn] = false
			self.usedIcons[icn] = true
		end
		if icn then
			SetRaidTarget(targetID, icon or icn or 0)
			if time or self.timeOut then
				Boss:Schedule(self.id, time or self.timeOut, hideIcon, self, targetID)
			end
			Boss:Schedule("ResetIcons", 2, iconProt.ResetIcons, self)
		end
	end
end
iconProt.Start = iconProt.Show

function iconProt:Hide(extraID)
	if not extraID then
		for id, icon in pairs(self.oldIcons) do
			SetRaidTarget(id, icon)
		end
		wipe(self.oldIcons)
		return
	end

	local targetID = Boss:UIDFromRandom(extraID)
	
	for id, icon in pairs(self.oldIcons) do
		if targetID == id then
			SetRaidTarget(id, icon)
			self.oldIcons[id] = nil
			break
		end
	end
end

function iconProt:Schedule(stime, info, ...)
	self.mod:Schedule(self.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function iconProt:ScheduleRepeating(stime, now, info, ...)
	self.mod:ScheduleRepeating(self.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.scheduled, self.id..(info or ""))
end

function iconProt:Unschedule(extraID)
	self.mod:Unschedule(self.id..(extraID or ""))
	RW:TRemove(self.scheduled, self.id..(extraID or ""))
end

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.ICON(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")	
	group:SetLayout("Flow")
	
	for bType, value in pairs(self.BITS.ICON) do
		if bit.band(self.options[option.text].ICON, value) == value then
			local check = AceGUI:Create("ImageCheckBox")
			check:SetRelativeWidth(0.2)
			check:SetValue(bit.band(self.db.profile[option.text].ICON, value) == value)
			check:SetUserData("option", option)
			check:SetUserData("type", "ICON")
			check:SetUserData("subType", bType)
			check:SetUserData("boss", self)
			check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)	
			check:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[bType]))
			group:AddChild(check)
		end
	end
	return group
end 